import os
import logging
import pandas as pd
from datetime import datetime, timedelta

class TimeFilter:
    """基于拍摄时间过滤图片"""

    def __init__(self, output_dir):
        self.log_file = os.path.join(output_dir, 'del_imgs.txt')
        self.logger = logging.getLogger('UAV_Preprocess.TimeFilter')
        self.time_threshold = timedelta(minutes=5)  # 5分钟阈值

    def filter_by_date(self, points_df: pd.DataFrame) -> pd.DataFrame:
        """根据拍摄时间过滤图片
        
        如果相邻两张图片的拍摄时间差超过5分钟，过滤掉后续所有图片
        
        Args:
            points_df: 包含图片信息的DataFrame，必须包含'file'和'date'列
            
        Returns:
            过滤后的DataFrame
        """
        self.logger.info("开始基于拍摄时间进行过滤")

        # 确保date列存在且不为空
        if 'date' not in points_df.columns:
            self.logger.error("输入数据中缺少date列")
            return points_df
        
        # 删除date为空的行
        points_df = points_df.dropna(subset=['date'])
        
        if len(points_df) == 0:
            self.logger.warning("没有有效的拍摄时间数据")
            return points_df

        # 按时间排序
        points_df = points_df.sort_values('date')
        self.logger.info(f"排序后的时间范围: {points_df['date'].min()} 到 {points_df['date'].max()}")

        # 计算时间差
        time_diffs = points_df['date'].diff()
        
        # 找到第一个时间差超过阈值的位置
        break_idx = None
        for idx, time_diff in enumerate(time_diffs):
            if time_diff and time_diff > self.time_threshold:
                break_idx = idx
                break_time = points_df.iloc[idx]['date']
                self.logger.info(f"在索引 {idx} 处发现时间断点，时间差为 {time_diff}")
                self.logger.info(f"断点时间: {break_time}")
                break

        # 如果找到断点，过滤掉后续图片
        if break_idx is not None:
            to_delete = points_df.iloc[break_idx:]['file'].tolist()
            self.logger.info(f"将删除 {len(to_delete)} 张断点后的图片")
            
            # 记录被删除的图片
            with open(self.log_file, 'a', encoding='utf-8') as f:
                for img in to_delete:
                    f.write(img + '\n')
                f.write('\n')
            
            # 保留断点之前的图片
            filtered_df = points_df.iloc[:break_idx]
            self.logger.info(f"时间过滤完成，保留了 {len(filtered_df)} 张图片")
            return filtered_df
        
        self.logger.info("未发现时间断点，保留所有图片")
        return points_df
